package de.hawhamburg.reachability.conf.resource;

import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import de.hawhamburg.reachability.util.XMLUtil;

/**
 * Simple access to common configuration variable encoded as XML configuration file.<br><br>
 * 
 * This class represents a common XML configuration file. It simplifies the access to the XML encoded configuration 
 * variables.  
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class SimpleXMLConfigFile {
    /**
     * The common xPath to the settings.
     */
    private String xSettings                = "/configuration/settings";
    
    /**
     * The common tag name of an configuration element.
     */
    private String varElementName           = "set";
    
    /**
     * The common attribute name of an configuration variable.
     */
    private String varKeyAttributeName      = "name";
    
    /**
     * The path which refers to the XML configuration file.
     */
    private String xmlFilePath              = null;
    
    /**
     * The local XML source as {@link Document}.
     */
    private Document localXMLSource         = null;
    
    /**
     * Creates a new {@link SimpleXMLConfigFile} object and initially loads the {@link Document} from file.
     * 
     * @param path          The path which refers to the XML configuration file.
     */
    public SimpleXMLConfigFile(final String path) {
        // Set file path
        xmlFilePath = path;
        
        // Load document
        refresh();
    }
    
    /**
     * Creates a new {@link SimpleXMLConfigFile} object, with dissent standard access structure, and initially loads the
     * {@link Document} from file.
     * 
     * @param path          The path which refers to the XML configuration file.
     * @param settingsX     The xPath to the settings.
     * @param elementX      The tag name of a configuration element.
     * @param attributeX    The name of the attibute which contains the name of the configuration variable.
     */
    protected SimpleXMLConfigFile(final String path,
                                  final String settingsX,
                                  final String elementX,
                                  final String attributeX) {
        // Set file path
        xmlFilePath         = path;
        
        // Alternative name settings...
        xSettings           = settingsX;
        varElementName      = elementX;
        varKeyAttributeName = attributeX;
        
        // Load document
        refresh();
    }
    
    /**
     * Fetches the value of a certain configuration variable from the XML document.
     * 
     * @param config        The name of the configuration variable as {@link String}.
     * @return              The value of the element if found as {@link String}.
     */
    public final String get(final String config) {
        // Ensure that the configuration has been loaded...
        checkLoad();
        
        // Lookup Variable
        String result = null;
        if (localXMLSource != null) {
            String xPath            = xSettings + "/" + varElementName
                                      + "[@" + varKeyAttributeName + "='"
                                      + config.trim().toUpperCase() + "']";
            List<Element> results   = XMLUtil.elementListByXPath(localXMLSource, xPath);
            if (results.size() > 0) {
                result = results.get(0).getTextContent();
                
                if (results.size() > 1) {
                    System.out.println("MORE THEN ONE DEFINITIONS OF '" + config + "' FOUND IN '" + xmlFilePath
                                       + "' ALL BUT THE FIRST IGNORED!");
                }
            } else {
                System.out.println("DEFINITION OF '" + config + "' NOT FOUND IN '" + xmlFilePath + "'");
            }
        } else {
            System.err.println("NO LOCAL XML SOURCE");
        }
        
        return result;
    }
    
    /**
     * Fetches the value of a certain configuration variable as {@link Integer} from the XML document.
     * 
     * @param config        The name of the configuration variable as {@link String}.
     * @return              The value of the element if found as {@link Integer}.
     */
    public final Integer getInteger(final String config) {
        String  value  = get(config);
        Integer result = null;
        
        try {
            result = Integer.valueOf(value);
        } catch (NumberFormatException e) {
            System.out.println("[NumberFormatException] UNABLE TO CONVERT INTO INTEGER FROM '" + value + "'");
        }
        
        return result;
    }

    /**
     * Fetches the value of a certain configuration variable as {@link Double} from the XML document.
     * 
     * @param config        The name of the configuration variable as {@link String}.
     * @return              The value of the element if found as {@link Double}.
     */
    public final Double getDouble(final String config) {
        String value  = get(config);
        Double result = null;
        
        try {
            result = Double.valueOf(value);
        } catch (NumberFormatException e) {
            System.out.println("[NumberFormatException] UNABLE TO CONVERT INTO DOUBLE FROM '" + value + "'");
        }
        
        return result;
    }
    
    /**
     * Fetches the value of a certain configuration variable as {@link Long} from the XML document.
     * 
     * @param config        The name of the configuration variable as {@link String}.
     * @return              The value of the element if found as {@link Long}.
     */
    public final Long getLong(final String config) {
        String value  = get(config);
        Long   result = null;
        
        try {
            result = Long.valueOf(value);
        } catch (NumberFormatException e) {
            System.out.println("[NumberFormatException] UNABLE TO CONVERT INTO LONG FROM '" + value + "'");
        }
        
        return result;
    }
    
    /**
     * Fetches the value of a certain configuration variable as {@link Boolean} from the XML document.
     * 
     * @param config        The name of the configuration variable as {@link String}.
     * @return              The value of the element if found as {@link Boolean}.
     */
    public final Boolean getBoolean(final String config) {
        String  value  = get(config);
        Boolean result = null;
        
        try {
            result = Boolean.valueOf(value);
        } catch (NumberFormatException e) {
            System.out.println("[NumberFormatException] UNABLE TO CONVERT INTO BOOLEAN FROM '" + value + "'");
        }
        
        return result;
    }
    
    /**
     * Loads the local XML source, if it's <code>null</code>, by using {@link #refresh()}. 
     */
    private void checkLoad() {
        if (localXMLSource == null) {
            refresh();
        }
    }
    
    /**
     * Loads the local XML source from file using {@link XMLUtil#loadDocument(String)}.
     */
    public final void refresh() {
        localXMLSource = XMLUtil.loadDocument(xmlFilePath);
    }
}
