package de.hawhamburg.reachability.ui.frame.resource;

import java.util.Map;
import java.util.HashMap;

import com.google.gson.Gson;

import de.hawhamburg.livingplace.messaging.activemq.wrapper.LPSubscriber;
import de.hawhamburg.reachability.StopableObject;
import de.hawhamburg.reachability.conf.MainConfig;
import de.hawhamburg.reachability.enumeration.ReachabilityGroup;
import de.hawhamburg.reachability.enumeration.ReachabilityStatus;
import de.hawhamburg.reachability.json.JsonReachabilityInformation;
import de.hawhamburg.reachability.ui.frame.ReachabilityUI;
import de.hawhamburg.reachability.util.ConnectionUtil;

/**
 * Updates the {@link ReachabilityUI} with new reachability status informations.<br><br>
 * 
 * This thread shall keep the {@link ReachabilityUI} up to date.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class ReachabilityUIUpdater extends Thread implements StopableObject {
    
    /**
     * The {@link ReachabilityUI} object, which shall be updated.
     */
    private ReachabilityUI ui       = null;

    /**
     * Determines if the method {@link #shutDown()} has been called.
     */
    private boolean shutDown        = false;
    
    /**
     * An {@link LPSubscriber}, which listens on the reachability status topic (<i>REACHABILITY_AGENT_RESULT_TOPIC</i>).
     */
    private LPSubscriber infoTopic  = null;
    
    /**
     * An hash map holding the last known states of reachability for each {@link ReachabilityGroup} 
     * item.
     */
    private Map<ReachabilityGroup, ReachabilityStatus> internStatus =
            new HashMap<ReachabilityGroup, ReachabilityStatus>();
    
    /**
     * Generates a new {@link ReachabilityUIUpdater} thread to keep the {@link ReachabilityUI} <i>UI</i> up to date.
     *  
     * @param sourceUi      A {@link ReachabilityUI}, which shall be updated.
     */
    public ReachabilityUIUpdater(final ReachabilityUI sourceUi) {
        this.ui = sourceUi;
    }

    /**
     * @see de.hawhamburg.reachability.StopableObject#shutDown()
     */
    public final void shutDown() {
        this.shutDown = true;
    }

    /**
     * @see de.hawhamburg.reachability.StopableObject#isActive()
     * 
     * @return              <code>true</code> if {@link #shutDown()} wasn't called, otherwise <code>false</code>.
     */
    public final boolean isActive() {
        return !this.shutDown;
    }

    /**
     * Connects to the reachability information topic using {@link #init()} and frequently tries to get the newest 
     * reachability states from this topic. If a new state arrives, the {@link ReachabilityUI} {@link #ui} will be 
     * updated using {@link ReachabilityUI#refresh(Map)}.
     */
    public final void run() {
        this.init();
        
        while (this.isActive()) {
            String msg = ConnectionUtil.getMessageOnTopic(infoTopic);
            if (msg != null) {
                Gson g = new Gson();
                JsonReachabilityInformation info = g.fromJson(msg, JsonReachabilityInformation.class);
                for (Map.Entry<ReachabilityGroup, ReachabilityStatus> state : info.currentStatus.entrySet()) {
                    if (this.internStatus.containsKey(state.getKey())) {
                        this.internStatus.put(state.getKey(), state.getValue());
                    }
                }
                this.ui.refresh(this.internStatus);
            }
        }
        
        this.end();
    }
    
    /**
     * Opens a connection the the reachability information topic (<i>REACHABILITY_AGENT_RESULT_TOPIC</i>) and 
     * initializes the intern state hash map {@link #internStatus} with {@link ReachabilityStatus#NOT_CALCULATED}.
     */
    public final void init() {
        // Get topics name
        String topicName = MainConfig.get("REACHABILITY_AGENT_RESULT_TOPIC");
        
        // Open topics
        infoTopic = ConnectionUtil.getSubscriber(topicName, this);
        
        // Preset status array
        for (ReachabilityGroup x : ReachabilityGroup.values()) {
            internStatus.put(x, ReachabilityStatus.NOT_CALCULATED);
        }
    }
    
    /**
     * Disconnects the {@link #infoTopic}, if it isn't <code>null</code>.
     */
    public final void end() {
        if (infoTopic != null) {
            infoTopic.disconnect();
        }
    }
}
